# Copyright (c) 2010 Outpost Embedded, LLC
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA. Any Outpost Embedded trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Outpost Embedded, LLC.

import os

from PyQt4 import QtCore, QtGui


def qt_string_to_unicode(s):
    return str(s.toUtf8()).decode('utf-8')


class CallbackList(object):
    callbacks = None

    def __init__(self):
        self.callbacks = []

    def add(self, fn):
        if fn in self.callbacks:
            raise ValueError(fn)
        self.callbacks.append(fn)

    def remove(self, fn):
        self.callbacks.remove(fn)

    def call(self, *args, **kwargs):
        for fn in self.callbacks:
            fn(*args, **kwargs)


class FunctionObject(QtCore.QObject):
    instances = []

    trigger_timer_start = QtCore.pyqtSignal()

    __timer = None

    def __init__(self, fn, thread = None):
        super(FunctionObject, self).__init__()
        self.fn = fn

        # Prevent GC:
        type(self).instances.append(self)

        self.__timer = QtCore.QTimer()

        if (thread is not None) and (self.thread() != thread):
            self.moveToThread(thread)

    def moveToThread(self, thread):
        super(FunctionObject, self).moveToThread(thread)
        self.__timer.moveToThread(thread)

    def schedule(self, when = 0, single_shot = True):
        self.__timer.setInterval(when)
        self.__timer.setSingleShot(single_shot)
        self.__timer.timeout.connect(self.call_fn)
        if single_shot:
            self.__timer.timeout.connect(
              lambda: type(self).instances.remove(self)
            )
        self.trigger_timer_start.connect(self.__timer.start)
        self.trigger_timer_start.emit()

    @QtCore.pyqtSlot()
    def call_fn(self):
        self.fn()

    @classmethod
    def call_in_thread(cls, fn, thread):
        cls(fn, thread).schedule()

    @classmethod
    def call_in_main_thread(cls, fn):
        cls.call_in_thread(fn, QtGui.QApplication.instance().thread())


class QThreadWithEventLoop(QtCore.QThread):
    def run(self):
        self.exec_()


def locate_data_file(path, flags = ()):
    from rapidwriter import distinfo

    if 'arch' in flags:
        if 'pkg' in flags:
            base_path = distinfo.install_data_arch_pkg
        else:
            base_path = distinfo.install_data_arch
    elif 'indep' in flags:
        if 'pkg' in flags:
            base_path = distinfo.install_data_indep_pkg
        else:
            base_path = distinfo.install_data_indep
    else:
        base_path = distinfo.install_data

    return os.path.join(base_path, path)
